/*
 * This is the keyboards LED driver for linux framebuffer terminals.
 * (It will not work properly on pseudo terminals .)
 * 
 * Be careful: Switching pure LED state (without changing keyboard behaviour)
 * will disable terminals feature to react on NUM, CAPS or SCROLL key properly.
 * (You need to call the "reset" command within the terminal.)
 * 
 * To not switch LEDs directly but over their functions instead, MACRO
 * "__KEYBOARD_C_229db984fe7b4d419c4c43c9ad0e2028_FEATURE_USEKBDSTATE"
 * is active by default.
 * 
 * Stephan Baerwolf, Ilmenau 2013
 */

#define __KEYBOARD_C_229db984fe7b4d419c4c43c9ad0e2028 1
#define __KEYBOARD_C_229db984fe7b4d419c4c43c9ad0e2028_FEATURE_USEKBDSTATE 1

#include "keyboard.h"

#include <stdlib.h>
#include <stdint.h>

#include <unistd.h>
#include <termios.h>
#include <poll.h>

#include <fcntl.h>
#include <limits.h>

// here it gets platform dependend (linux)
#include <linux/kd.h>
#include <sys/ioctl.h>

static uint64_t initialized = 0;
static struct termios oldt, newt;
static char     initialledstate;
static char     initialkbdstate;

// from setleds (sys-apps/kbd-1.15.3)
static int __getleds(char *cur_leds) {
    if (ioctl(STDIN_FILENO, KDGETLED, cur_leds)) return 1;
    return 0;
}

static int __setleds(char cur_leds) {
    if (ioctl(STDIN_FILENO, KDSETLED, cur_leds)) return 1;
    return 0;
}

static int __getflags(char *cur_leds) {
    if (ioctl(STDIN_FILENO, KDGKBLED, cur_leds)) return 1;
    return 0;
}

static int __setflags(char cur_leds) {
    if (ioctl(STDIN_FILENO, KDSKBLED, cur_leds)) return 1;
    return 0;
}


int keyboard_isinitialized(void) {
  return (initialized>0);
}

int keyboard_initialize(void) {
  if (initialized==0) {

    // do sth. here:
    if (__getflags(&initialkbdstate)!=0) return -2;
    if (__getleds(&initialledstate)!=0) return -1;

    tcgetattr(STDIN_FILENO, &oldt);
    newt = oldt;
    newt.c_lflag &= ~(ICANON | ECHO);
    tcsetattr(STDIN_FILENO, TCSANOW, &newt);

    initialized=1;
    return 0;
  }
  initialized++;
  return 1;
}

int keyboard_finalize(void) {
  if (initialized > 0) {
    initialized--;
    if (initialized==0) {
      
      // undo sth. here:
      __setflags(initialkbdstate);
#ifndef __KEYBOARD_C_229db984fe7b4d419c4c43c9ad0e2028_FEATURE_USEKBDSTATE
      __setleds(initialledstate);
#endif
      tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
      
      return 0;
    } else return 1;
  }
  return 2;
}

int keyboard_keypressed(void) {
  int result;
  struct pollfd poll_stdin[1];
  poll_stdin[0].fd	= STDIN_FILENO;
  poll_stdin[0].events	= POLLIN;
  result=poll(poll_stdin, 1, 0);
  if (result>0) {
    if (poll_stdin[0].revents & POLLIN) {
      return !0;
    }
  } 
  return 0;
}

size_t keyboard_clearkeybuffer(void) {
  size_t result = 0;
  char buffer;

  while (keyboard_keypressed()) {
    result+=read(STDIN_FILENO, &buffer, 1);
  }

  return result;
}

static char __tglleds_from_leds(char cur_leds, int leds) {
  char result = cur_leds;
#ifdef __KEYBOARD_C_229db984fe7b4d419c4c43c9ad0e2028_FEATURE_USEKBDSTATE
  if (leds & KEYBOARD_LED_NUMLOCK)	result ^= K_NUMLOCK;
  if (leds & KEYBOARD_LED_CAPSLOCK)	result ^= K_CAPSLOCK;
  if (leds & KEYBOARD_LED_SCROLLLOCK)	result ^= K_SCROLLLOCK;
#else
  if (leds & KEYBOARD_LED_NUMLOCK)	result ^= LED_NUM;
  if (leds & KEYBOARD_LED_CAPSLOCK)	result ^= LED_CAP;
  if (leds & KEYBOARD_LED_SCROLLLOCK)	result ^= LED_SCR;
#endif
  return result;
}

int keyboard_setled(int leds, int state) {
  char rcvleds, sendleds_leftterm, sendleds_rightterm;
#ifdef __KEYBOARD_C_229db984fe7b4d419c4c43c9ad0e2028_FEATURE_USEKBDSTATE
  if (__getflags(&rcvleds) != 0) return 2;
#else
  if (__getleds(&rcvleds) != 0) return 2;
#endif
  
  sendleds_leftterm	= rcvleds & (__tglleds_from_leds(0,state|(~leds)));
  sendleds_rightterm	= __tglleds_from_leds(0,state&leds);

#ifdef __KEYBOARD_C_229db984fe7b4d419c4c43c9ad0e2028_FEATURE_USEKBDSTATE
  return __setflags(sendleds_leftterm | sendleds_rightterm);
#else
  return __setleds(sendleds_leftterm | sendleds_rightterm);
#endif
}

int keyboard_toggleled(int leds) {
  char rcvleds;
#ifdef __KEYBOARD_C_229db984fe7b4d419c4c43c9ad0e2028_FEATURE_USEKBDSTATE
  if (__getflags(&rcvleds) != 0) return 2;
  return __setflags(__tglleds_from_leds(rcvleds,leds));
#else
  if (__getleds(&rcvleds) != 0) return 2;
  return __setleds(__tglleds_from_leds(rcvleds,leds));
#endif
}
