/*
 * This file implements a simple (and static) version
 * of a SPI Master in software.
 * As physical layer the keyboard LEDs are used:
 * 	SS/CS (chip select)	:  SCROLL LOCK LED
 * 	MOSI			:  CAPS LOCK LED
 * 	CLOCK			:  NUM LOCK LED
 * 
 * As input deals any key pressed: (use a optocoupler)
 * 	MISO == 1	<=>	any regular key pressed
 * (	MISO == 0	<=>	NO key pressed		)
 * 
 * Stephan Baerwolf, Ilmenau 2013
 */

#define __SOFTSPI_C_229db984fe7b4d419c4c43c9ad0e2028 1
#include "softspi.h"





#define __SOFTSPI_C_229db984fe7b4d419c4c43c9ad0e2028_USLEEPPERIOD 25000
// #define __SOFTSPI_C_229db984fe7b4d419c4c43c9ad0e2028_USLEEPPERIOD 50000
// #define __SOFTSPI_C_229db984fe7b4d419c4c43c9ad0e2028_USLEEPPERIOD 250000 /*debug purpose*/

static uint64_t initialized				=	0;

static uint32_t softspi_quarterperiodclockdelay_us	=	__SOFTSPI_C_229db984fe7b4d419c4c43c9ad0e2028_USLEEPPERIOD;
static uint8_t softspi_complementmask			=	0;



static uint8_t __softspi_tick(uint8_t bit) {
  // clock down and HALFwait
  keyboard_setled(SOFTSPI_CLK, 0 ^ softspi_complementmask); 
  keyboard_clearkeybuffer();
  usleep(softspi_quarterperiodclockdelay_us>>1);

  // prepare physical (and wait)
  if (bit) keyboard_setled(SOFTSPI_MOSI, SOFTSPI_MOSI ^ softspi_complementmask);
  else	   keyboard_setled(SOFTSPI_MOSI, 0            ^ softspi_complementmask);
  keyboard_clearkeybuffer();
  usleep(softspi_quarterperiodclockdelay_us);

  // clock up and wait until sample
  keyboard_setled(SOFTSPI_CLK, SOFTSPI_CLK ^ softspi_complementmask);
  keyboard_clearkeybuffer();
  usleep(softspi_quarterperiodclockdelay_us);

  // sample (and wait)
  if (softspi_complementmask & SOFTSPI_MISO)	bit=keyboard_keypressed()?0:1;
  else						bit=keyboard_keypressed()?1:0;
  keyboard_clearkeybuffer();
  usleep(softspi_quarterperiodclockdelay_us);

  // clock down and HALFwait
  keyboard_setled(SOFTSPI_CLK, 0 ^ softspi_complementmask); 
  keyboard_clearkeybuffer();
  usleep(softspi_quarterperiodclockdelay_us>>1);

  return bit;
}

uint8_t softspi_kick(uint8_t tx) {
  int i;
  uint8_t result = 0;

  //MSB first
  for (i=0;i<8;i++) {
    result<<=1;
    if (__softspi_tick(tx&0x80)) result+=1;
    tx<<=1;
  }

  return result;
}

int softspi_isinitialized(void) {
  return ((initialized>0)&&(keyboard_isinitialized()));
}

int softspi_initialize(void) {
  if (initialized==0) {

    // do sth. here:
    if (keyboard_initialize()<0) return -1;
    keyboard_setled(SOFTSPI_OUTS, SOFTSPI_SS ^ softspi_complementmask); // nothing to select

    initialized=1;
    return 0;
  }
  initialized++;
  return 1;
}

int softspi_finalize(void) {
  if (initialized > 0) {
    initialized--;
    if (initialized==0) {
      
      // undo sth. here:
      keyboard_clearkeybuffer();
      keyboard_finalize();
      
      return 0;
    } else return 1;
  }
  return 2;
}

uint32_t softspi_getclock(void) {
  return softspi_quarterperiodclockdelay_us;
}

int softspi_setclock(uint32_t clockdelay) {
  softspi_quarterperiodclockdelay_us=clockdelay;
  return 0;
}

uint8_t softspi_getcomplementmask(void) {
  return softspi_complementmask;
}

int softspi_setcomplementmask(uint8_t complement) {
  softspi_complementmask=complement;
  return 0;
}

void softspi_enable(void) {
  keyboard_setled(SOFTSPI_SS, 0 ^ softspi_complementmask);
}

void softspi_disable(void) {
  keyboard_setled(SOFTSPI_SS, SOFTSPI_SS ^ softspi_complementmask);
}
